import { BleScanMode } from "../common";
import { enableLocationRequest, isEnabled } from "@nativescript/geolocation";
import { hasPermission, requestPermission } from "nativescript-permissions";
import { Application, Utils } from "@nativescript/core";
import { requestEnableBluetooth } from "./helpers";
import { from, map, Observable, switchMap } from "rxjs";
import { scanResultsToBleScanResults } from "./parsers";
export class AndroidBleScanAdapter {
    constructor(bluetoothAdapter = Utils.android
        .getApplicationContext()
        .getSystemService(android.content.Context.BLUETOOTH_SERVICE).getAdapter(), sdkVersion = android.os.Build.VERSION.SDK_INT, activityGetter = () => Application.android.foregroundActivity) {
        this.bluetoothAdapter = bluetoothAdapter;
        this.sdkVersion = sdkVersion;
        this.activityGetter = activityGetter;
    }
    async isReady() {
        const locFeaturesReady = await isEnabled();
        if (!locFeaturesReady) {
            return false;
        }
        if (this.sdkVersion >= 31 &&
            !hasPermission(android.Manifest.permission.BLUETOOTH_SCAN)) {
            return false;
        }
        return this.bluetoothAdapter.isEnabled();
    }
    async prepare() {
        const locFeaturesReady = await isEnabled();
        if (!locFeaturesReady) {
            await enableLocationRequest(false, true);
        }
        if (this.sdkVersion >= 31 &&
            !hasPermission(android.Manifest.permission.BLUETOOTH_SCAN)) {
            await requestPermission(android.Manifest.permission.BLUETOOTH_SCAN);
        }
        if (!this.bluetoothAdapter.isEnabled()) {
            await requestEnableBluetooth(this.activityGetter(), this.bluetoothAdapter);
        }
    }
    streamBleScanResults(reportDelay, scanMode) {
        return from(this.isReady()).pipe(map((ready) => {
            if (!ready)
                throw new Error("Could not perform BLE scan. Insufficient permissions, disabled location services or bluetooth adapter. Please, call prepare() first before requesting a BLE scan");
        }), switchMap(() => this.createBleScanStream(reportDelay, scanMode)));
    }
    createBleScanStream(reportDelay, scanMode) {
        return new Observable((subscriber) => {
            const scanner = no.nordicsemi.android.support.v18.scanner.BluetoothLeScannerCompat.getScanner();
            const settingsBuilder = new no.nordicsemi.android.support.v18.scanner.ScanSettings.Builder()
                .setScanMode(nativeScanModeFrom(scanMode))
                .setUseHardwareBatchingIfSupported(false);
            if (reportDelay > 0) {
                settingsBuilder.setReportDelay(reportDelay);
            }
            const settings = settingsBuilder.build();
            const scanCallback = new BleScanCallback((results) => {
                subscriber.next(scanResultsToBleScanResults(results));
            }, (error) => subscriber.error(new Error(`BleScanAdapter ERROR: ${error}`)));
            scanner.startScan(null, settings, scanCallback);
            return () => {
                scanner.stopScan(scanCallback);
            };
        });
    }
}
function nativeScanModeFrom(mode) {
    switch (mode) {
        case BleScanMode.LOW_POWER:
            return no.nordicsemi.android.support.v18.scanner.ScanSettings
                .SCAN_MODE_LOW_POWER;
        case BleScanMode.LOW_LATENCY:
            return no.nordicsemi.android.support.v18.scanner.ScanSettings
                .SCAN_MODE_LOW_LATENCY;
        case BleScanMode.BALANCED:
            return no.nordicsemi.android.support.v18.scanner.ScanSettings
                .SCAN_MODE_BALANCED;
    }
}
var BleScanCallback = /** @class */ (function (_super) {
    __extends(BleScanCallback, _super);
    function BleScanCallback(onResults, onError) {
        var _this = _super.call(this) || this;
        _this.onResults = onResults;
        _this.onError = onError;
        // @ts-ignore
        return global.__native(_this);
    }
    BleScanCallback.prototype.onScanResult = function (_callbackType, result) {
        this.onResults(java.util.Arrays.asList([result]));
    };
    BleScanCallback.prototype.onBatchScanResults = function (results) {
        this.onResults(results);
    };
    BleScanCallback.prototype.onScanFailed = function (errorCode) {
        this.onError(new Error("BLE scan failed with error code: ".concat(errorCode)));
    };
    return BleScanCallback;
}(no.nordicsemi.android.support.v18.scanner
    .ScanCallback));
//# sourceMappingURL=adapter.js.map